 /*****************************************************************************
* Filename          : SERIAL.c $workfile: $
*
* Module            : 
*
* Author            : V A TArrant
* Description       : This module holds initialisation and driver routines for
*					  the serial interface  
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   1.0     06Feb2008   Val Tarrant     First Issue
*
*   $Log: serial.c,v $
*   Revision 1.1  2009/09/22 15:54:23  rt
*   Initial commit to CVS.
*   Originally version 00.00.07
*   Committed on the Free edition of March Hare Software CVSNT Server.
*   Upgrade to CVS Suite for more features and support:
*   http://march-hare.com/cvsnt/
*
*
*
* Copyright         : Cambridge Micro Engineering, feb 2008
*
******************************************************************************/

/*****************************************************************************
* Library Includes
*****************************************************************************/
#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#include <libpic30.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#include <libpic24.h>
#endif

#include 	<uart.h>
#include	<stdio.h>

/*****************************************************************************
* Project Includes
*****************************************************************************/

#include    "serial.h"
#include	"typedefs.h"

/******************************************************************************
*   Local Constants
******************************************************************************/

//#define SERIAL_INTERRUPT

#define MAX_SERIAL_BUFFER_SIZE  256 //64 

/*****************************************************************************
*   Global Variables
******************************************************************************/

/* 	buffer to store data to be printed used by	ISR - hence volatile  */
volatile byte    SERIALOutBuffer[ MAX_SERIAL_BUFFER_SIZE];

/* The index to the next free element in buffer */
volatile byte    iSERIALOutBufferIn= 0;    

/* Used to idicate if there are any data to be sent */
volatile BOOLEAN    SERIALOutBufferEmpty = TRUE;

/* Used to idicate if the buffer is full */
volatile BOOLEAN    SERIALOutBufferFull = FALSE;

/* The index to the next value to be taken from Buffer */
volatile byte    iSERIALOutBufferOut= 0; 

/* used to indicate buffer may be used to send serial data */
static BOOLEAN	SERIALOutBufferOpen = TRUE;


/* 	buffer to store data from ISR - hence volatile  */
volatile byte    SERIALInBuffer[ MAX_SERIAL_BUFFER_SIZE];

/* The index to the next free element in buffer */
volatile byte    iSERIALInBufferIn= 0;    

/* Used to idicate if there are any data to be sent */
volatile BOOLEAN    SERIALInBufferEmpty = TRUE;

/* Used to idicate if the buffer is full */
volatile BOOLEAN    SERIALInBufferFull = FALSE;

/* The index to the next value to be taken from Buffer */
volatile byte    iSERIALInBufferOut= 0; 

/* used to indicate buffer may be used to send serial data */
static BOOLEAN	SERIALInBufferOpen = TRUE;

/* used to indicate end of transmission on last byte in buffer */
volatile BOOLEAN TxEnded = TRUE;

/*****************************************************************************
*   Local Function Prototypes
*****************************************************************************/

/*****************************************************************************
*   Exported functions
*****************************************************************************/

/*****************************************************************************
*
* Name:    SERIALInit
*
* Description: 		Serial settings are:
*					Baud rate 57600
*					Data bits 8
*					Stop bits 1
*					Parity    None
*
*****************************************************************************/
void SERIALInit (void)
{

//#define MIDI_DEBUG

    DisableIntU2RX;/* Make sure UART Interrupts are disabled */
    DisableIntU2TX; 
    
    TxEnded = TRUE;

#ifdef MIDI_DEBUG
    ODCFbits.ODCF5 = 1;  /* Open drain */
#endif

   CloseUART2();
	
   OpenUART2(	UART_EN & UART_IDLE_CON & UART_IrDA_DISABLE & // config1
                UART_MODE_FLOW & UART_UEN_00 & UART_DIS_WAKE & 
                UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_UXRX_IDLE_ONE & 
                UART_NO_PAR_8BIT & UART_BRGH_SIXTEEN & UART_1STOPBIT,
                UART_INT_TX_BUF_EMPTY & UART_INT_TX &   // config2
                UART_TX_ENABLE & UART_IrDA_POL_INV_ZERO &
                UART_SYNC_BREAK_DISABLED & UART_INT_RX_CHAR,
                //78                                        // Baudrate  ((79227500/2) / (16*31250)) - 1
                42                                        // Baudrate  ((79227500/2) / (16*56700)) - 1
             );

    ConfigIntUART2(UART_RX_INT_DIS & UART_RX_INT_PR6 &  
                   UART_TX_INT_DIS & UART_TX_INT_PR2);

	U2STAbits.UTXISEL0 = 1;	/* Interrupt when the last character is shifted out of the Transmit Shift Register; all transmit operations are complete */
	U2STAbits.UTXISEL1 = 0;	

	/* clear interrupt flags  */
	IFS1bits.U2TXIF = 0;
	IFS1bits.U2RXIF = 0;

    int RxCh = U2RXREG;	    /* Read character */

#ifdef SERIAL_INTERRUPT
    EnableIntU2RX;
#endif

	iSERIALOutBufferIn= 0;   
	SERIALOutBufferEmpty = TRUE;
	SERIALOutBufferFull = FALSE;
	iSERIALOutBufferOut= 0; 
	SERIALOutBufferOpen = TRUE;

	iSERIALInBufferIn= 0;    
	SERIALInBufferEmpty = TRUE;
	SERIALInBufferFull = FALSE;
	iSERIALInBufferOut= 0; 
	SERIALInBufferOpen = TRUE;
	TxEnded = TRUE;

}
/*****************************************************************************
*
* Name:    SERIALReset
*
* Description: 	reset serial setting 
*
*****************************************************************************/
void SERIALReset (void)
{
    /* Reset uart */
	U2STA  = 0x0000; /* Reset status register and disable TX */
	U2MODE = 0x0000; /* Reset UART to power on default */
	U2BRG = 0;      /*  Reset BAUD Rate Setting of Uart */

    CloseUART2();
}

/*****************************************************************************
*
* Name:         SERIALCloseOutBuffer
*
* Description:   Close out buffer to new input
*          
*****************************************************************************/
void SERIALCloseOutBuffer (void)
{
	SERIALOutBufferOpen = FALSE;
}

/*****************************************************************************
*
* Name:         SERIALCloseInBuffer
*
* Description:   Close in buffer to new input
*
*****************************************************************************/
void SERIALCloseInBuffer (void)
{
	SERIALInBufferOpen = FALSE;
}



/*****************************************************************************
*
* Name:         SERIALOpenOutBuffer
*
* Description:   Open out buffer to new input
*          
*****************************************************************************/
void SERIALOpenOutBuffer (void)
{
	SERIALOutBufferOpen = TRUE;
}

/*****************************************************************************
*
* Name:         SERIALOpenInBuffer
*
* Description:   Open in buffer to new input
*          
*****************************************************************************/
void SERIALOpenInBuffer (void)
{
	SERIALInBufferOpen = TRUE;
}

/*****************************************************************************
*
* Name:         SERIALInBufferEmpty
*
* Description:   check to see if data in input buffer
*          
*****************************************************************************/
BOOLEAN SERIALInBufferIsEmpty (void)
{

    if (SERIALInBufferEmpty)
        return TRUE;
	else
        return FALSE;
}

/*****************************************************************************
*
* Name:         SERIALOutBufferEmpty
*
* Description:   check to see if data in output buffer
*          
*****************************************************************************/
BOOLEAN SERIALOutBufferIsEmpty (void)
{
    if (SERIALOutBufferEmpty)
        return TRUE;
	else
        return FALSE;
}

/*****************************************************************************
*
* Name:         SERIALWriteByte
*
* Description:   Output character to USART output stream
*          
*          
*          
*
*****************************************************************************/
void SERIALWriteByte (byte c)
{
	/* if buffer full ignore */
	if (!SERIALOutBufferFull)
    {   /* if buffer open write byte into buffer */
	    if (SERIALOutBufferOpen)
	    {
		    /* disable interrupts   */
     		DisableIntU2TX;

	    	/* Save char to buffer to be dealt with by serial ISR  */
		    SERIALOutBuffer[iSERIALOutBufferIn] = c;

    		/* increment pointer and wrap if past the end of the buffer */
        	iSERIALOutBufferIn = ((++iSERIALOutBufferIn)  & (MAX_SERIAL_BUFFER_SIZE-1));

		    /* look for buffer full*/
    		if (iSERIALOutBufferIn == iSERIALOutBufferOut)
	    		SERIALOutBufferFull = TRUE;

            /* if buffer currently empty force an interrupt when enabled */            
            if (TxEnded)
	        {    
                IFS1bits.U2TXIF = 1;
                TxEnded = FALSE;
            }

		    /* buffer not empty */
        	SERIALOutBufferEmpty = FALSE;

	    	/* enable interrupts   */
#ifdef SERIAL_INTERRUPT
 		    EnableIntU2TX;
#endif
    	}	/* if SERIALInBufferOpen*/	    
    }/* else ignore character */
}



/*****************************************************************************
*
* Name:         SERIALReadByte
*
* Description:   Output character to USART output stream
*          
*          
*          
*
*****************************************************************************/
byte  SERIALReadByte (void)
{

    static byte Temp = 0;
  
    /* disable interrupts   */
    DisableIntU2RX;

    /* get char from buffer to be dealt with  */
    byte c = SERIALInBuffer[iSERIALInBufferOut];

//        if ((iSERIALInBufferOut == iSERIALInBufferIn)&&(SERIALInBufferEmpty == FALSE))
//        {
//            Temp++;
//        }

    if (c == 0)
    {
        if (Temp == 0)
            Temp++;
        else
            ClrWdt();
    }

        ClrWdt();

    /* increment pointer and wrap if past the end of the buffer */
    iSERIALInBufferOut = ((++iSERIALInBufferOut)  & (MAX_SERIAL_BUFFER_SIZE-1));

    /* look for buffer empty */
    if (iSERIALInBufferIn == iSERIALInBufferOut)
        SERIALInBufferEmpty = TRUE;

    /* buffer not full */
    SERIALInBufferFull = FALSE;

   	/* enable interrupts   */
#ifdef SERIAL_INTERRUPT
    EnableIntU2RX;
#endif

    return c;

}
	


/*****************************************************************************
*
* Name:    SERIAL Interrupt
*
* Description: ISR for serial TX interrupt
*          
*          
*          
*
*****************************************************************************/

#ifdef SERIAL_INTERRUPT
void    __attribute__((interrupt,auto_psv)) _U2TXInterrupt(void)
#else
void    U2TXInterrupt(void)
#endif
{

	if (SERIALOutBufferEmpty)
	{
		/* clear interrupt flag  */
		IFS1bits.U2TXIF = 0;

		/* disable interrupts   */
	 	DisableIntU2TX;
        TxEnded = TRUE;
	}
	else
	{	
#ifndef SERIAL_INTERRUPT
		/* wait for TX buffer to empty  */
		if (( U2STAbits.TRMT  == 1 )&&( U2STAbits.UTXBF  == 0 ))
		{
#endif
		/* clear interrupt flag  */
		IFS1bits.U2TXIF = 0;	

		/* write next char from serial buffer */ 
		WriteUART2(	(unsigned int)(SERIALOutBuffer[iSERIALOutBufferOut]));

		/* increment out pointer and wrap  */
		iSERIALOutBufferOut = ((++iSERIALOutBufferOut)  & (MAX_SERIAL_BUFFER_SIZE-1));

		if (iSERIALOutBufferOut == iSERIALOutBufferIn)
			SERIALOutBufferEmpty = TRUE;

		SERIALOutBufferFull = FALSE;	
#ifndef SERIAL_INTERRUPT
		}
#endif				
	}

}   /* end of SERIALInterrupt */



/*****************************************************************************
*
* Name:    SERIAL Interrupt
*
* Description: ISR for serial RX interrupt
*          
*          
*          
*
*****************************************************************************/

#ifdef SERIAL_INTERRUPT
void    __attribute__((interrupt,auto_psv)) _U2RXInterrupt(void)
#else
void    U2RXInterrupt(void)
#endif
{
    byte c;
    static byte Temp = 0;

	/* clear interrupt flag  */
	IFS1bits.U2RXIF = 0;

    if (!SERIALInBufferFull)
    {
	    /* write next char from serial buffer */ 
	    c = SERIALInBuffer[iSERIALInBufferIn] = U2RXREG;
 

     if (c == 0)
    {
        if (Temp == 0)
            Temp++;
        else
            ClrWdt();
    }
            ClrWdt();

//        if (iSERIALInBufferOut == 96)
//        {
//            Temp++;
//        }

	    /* increment out pointer and wrap  */
	    iSERIALInBufferIn = ((++iSERIALInBufferIn)  & (MAX_SERIAL_BUFFER_SIZE-1));

	    if ( iSERIALInBufferIn == iSERIALInBufferOut)
		    SERIALInBufferFull = TRUE;

	    SERIALInBufferEmpty = FALSE;			
    }
    else
    {   /* read and discard */
        c = U2RXREG;
    }
        
}   /* end of SERIALInterrupt */

/*****************************************************************************
* End of file
*****************************************************************************/

