/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        adcDrv1.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC33Fxxxx
* Compiler:        MPLAB C30 v3.00 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all ownership and 
* intellectual property rights in the code accompanying this message and in all 
* derivatives hereto.  You may use this code, and any derivatives created by 
* any person or entity by or on your behalf, exclusively with Microchip,s 
* proprietary products.  Your acceptance and/or use of this code constitutes 
* agreement to the terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS".  NO 
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
* TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIP,S 
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE, WHETHER 
* IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), 
* STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, 
* PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF 
* ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN 
* ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT 
* ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO 
* THIS CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO 
* HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and 
* determining its suitability.  Microchip has no obligation to modify, test, 
* certify, or support the code.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Settu D 			07/09/06  First release of source file
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
**********************************************************************/

#if defined(__dsPIC33F__)
#include "p33fxxxx.h"
#elif defined(__PIC24H__)
#include "p24hxxxx.h"
#endif

#include "typedefs.h"
#include "adcDrv1.h"
#include "main.h"
#include "timers.h"
#include "keybrd.h"

/* Define Buffer Length for ADC1 */
#define  SAMP_BUFF_SIZE	 		1		/* Size of the input buffer per analog input */
#define  NUM_CHS2SCAN			14		/* Number of channels enabled for channel scan */

#define  NUM_SCANS 2 					/* Number of keyboard scan buffers required */

tADCValue AdcThreshold[NUM_ROWS] = { 500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500 };

/* Number of locations for ADC buffer = 14 (AN0 to AN13) x 1 = 14 words */
/* Align the buffer to 128 words or 256 bytes. This is needed for peripheral indirect mode */
tADCValue BufferA[NUM_CHS2SCAN] __attribute__((space(dma),aligned(256)));
tADCValue BufferB[NUM_CHS2SCAN] __attribute__((space(dma),aligned(256)));

unsigned char FillScan = 0;

unsigned char DmaBuffer = 0;

unsigned int ActiveRow = 0;

void ProcessADCSamples(unsigned int * AdcBuffer, unsigned int Row);

/*=============================================================================
ADC Initialisation for Channel Scan 
=============================================================================*/
void initAdc1(void)
{
	AD1CON1bits.FORM   = 0;		// Data Output Format: integer
	AD1CON1bits.SSRC   = 2;		// Sample Clock Source: GP Timer starts conversion
	AD1CON1bits.ASAM   = 1;		// ADC Sample Control: Sampling begins immediately after conversion
	AD1CON1bits.AD12B  = 0;		// 10-bit ADC operation
	

	AD1CON2bits.CSCNA = 1;		// Scan Input Selections for CH0+ during Sample A bit
	AD1CON2bits.CHPS  = 0;		// Converts CH0

	AD1CON3bits.ADRC = 0;		// ADC Clock is derived from Systems Clock
	AD1CON3bits.ADCS = 3;		// ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*3 = 101ns (~10Mhz)
								// ADC Conversion Time for 10-bit Tc=12*Tad = 1.2us	(825KHz)



	AD1CON1bits.ADDMABM = 0; 	// DMA buffers are built in scatter/gather mode
	AD1CON2bits.SMPI    = (NUM_CHS2SCAN-1);	// Number of ADC Channel scanned
	AD1CON4bits.DMABL   = 0;	// Each buffer contains 1 word

	//AD1CSSH/AD1CSSL: A/D Input Scan Selection Register
	AD1CSSH = 0x0000;
    AD1CSSL = 0x3FFF; /*  AN0 to AN13 */
 
 	//AD1PCFGH/AD1PCFGL: Port Configuration Register
	AD1PCFGL=0xC000;    /* ANO to AN13 as analogue inputs */
	AD1PCFGH=0xFFFF;

    //TRISB: Port direction bits for all analogue inputs must be set
    TRISB = 0x3FFF;             /* Leave AN14 and AN15 as Outputs */
    //LATB: Prime LATB to drive all Columns low
    LATB = 0x0000;

    //LATD: Drive all rows off
    LATD = 0;
    TRISD = 0x0000;  
	
	IFS0bits.AD1IF   = 0;		// Clear the A/D interrupt flag bit
	IEC0bits.AD1IE   = 0;		// Do Not Enable A/D interrupt 
	AD1CON1bits.ADON = 1;		// Turn on the A/D converter


}

void resetAdc1(void)
{
	AD1CON1bits.FORM   = 0;		// Data Output Format: integer
	AD1CON1bits.SSRC   = 2;		// Sample Clock Source: GP Timer starts conversion
	AD1CON1bits.ASAM   = 1;		// ADC Sample Control: Sampling begins immediately after conversion
	AD1CON1bits.AD12B  = 0;		// 10-bit ADC operation
	

	AD1CON2bits.CSCNA = 1;		// Scan Input Selections for CH0+ during Sample A bit
	AD1CON2bits.CHPS  = 0;		// Converts CH0

	AD1CON3bits.ADRC = 0;		// ADC Clock is derived from Systems Clock
	AD1CON3bits.ADCS = 3;		// ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*3 = 101ns (~10Mhz)
								// ADC Conversion Time for 10-bit Tc=12*Tad = 1.2us	(825KHz)



	AD1CON1bits.ADDMABM = 0; 	// DMA buffers are built in scatter/gather mode
	AD1CON2bits.SMPI    = (NUM_CHS2SCAN-1);	// Number of ADC Channel scanned
	AD1CON4bits.DMABL   = 0;	// Each buffer contains 1 word

	//AD1CSSH/AD1CSSL: A/D Input Scan Selection Register
	AD1CSSH = 0x0000;
    AD1CSSL = 0x3FFF; /*  AN0 to AN13 */
 
 	//AD1PCFGH/AD1PCFGL: Port Configuration Register
	AD1PCFGL=0xC000;    /* ANO to AN13 as analogue inputs */
	AD1PCFGH=0xFFFF;

    //TRISB: Port direction bits for all analogue inputs must be set
    TRISB = 0x3FFF;             /* Leave AN14 and AN15 as Outputs */
    //LATB: Prime LATB to drive all Columns low
    LATB = 0x0000;

    //LATD: Drive all rows off
    LATD = 0;
    TRISD = 0x0000;  
	
	IFS0bits.AD1IF   = 0;		// Clear the A/D interrupt flag bit
	IEC0bits.AD1IE   = 0;		// Do Not Enable A/D interrupt 
	AD1CON1bits.ADON = 0;		// Turn on the A/D converter


}



void startAdc1(void)
{
    //LATD: Drive active row 
    TRISD = 0x0000;     
    LATD = Bit(ActiveRow);
 
    TIMER3Init() ;  /* moved from  main  initialisation */
}

// DMA0 configuration
// Direction: Read from peripheral address 0-x300 (ADC1BUF0) and write to DMA RAM 
// AMODE: Peripheral Indirect Addressing Mode
// MODE: Continuous, Ping-Pong Mode
// IRQ: ADC Interrupt

void initDma0(void)
{
    DmaBuffer = 0;

	DMA0CONbits.AMODE = 2;			// Configure DMA for Peripheral indirect mode
	DMA0CONbits.MODE  = 2;			// Configure DMA for Continuous Ping-Pong mode
	DMA0PAD=(int)&ADC1BUF0;
	DMA0CNT = (SAMP_BUFF_SIZE*NUM_CHS2SCAN)-1;					
	DMA0REQ = 13;					// Select ADC1 as DMA Request source

	DMA0STA = __builtin_dmaoffset(&BufferA[0]);		
	DMA0STB = __builtin_dmaoffset(&BufferB[0]);

	IFS0bits.DMA0IF = 0;			//Clear the DMA interrupt flag bit
    IEC0bits.DMA0IE = 1;			//Set the DMA interrupt enable bit

	DMA0CONbits.CHEN=1;				// Enable DMA

}

void resetDma0(void)
{
	IFS0bits.DMA0IF = 0;			//Clear the DMA interrupt flag bit
    IEC0bits.DMA0IE = 0;			//clear the DMA interrupt enable bit

	DMA0CONbits.CHEN=0;				// disable DMA


}

/*=============================================================================
_DMA0Interrupt(): ISR name is chosen from the device linker script.
=============================================================================*/

void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
{
    /* Turn off timer - wont then kick off new ADC cycle */
    TIMER3Stop();
    
    /* Disharge analogue input lines to ground by changing to digital outputs */
 	/* Set AD1PCFGH/AD1PCFGL: Port Configuration Register */
	AD1PCFGL=0xFFFF;    /* All ANx digital inputs */
	AD1PCFGH=0xFFFF;

    /* Drive all Rows off */
    LATD = 0x0000;

    /* Port direction bits for all analogue inputs must be set now to outputs */
    TRISB = 0x0000;             /* All Ports to Outputs (LATD already zero'd) */

    /* Allow time for discharge */
    Nop();
    Nop();
    Nop();
    Nop();


    /* Reset Analogue lines */
 	/* AD1PCFGH/AD1PCFGL: Port Configuration Register */
	AD1PCFGL=0xC000;    /* ANO to AN13 as analogue inputs */
	AD1PCFGH=0xFFFF;

    /* TRISB: Port direction bits for all analogue inputs must be set */
    TRISB = 0x3FFF;             /* Leave AN14 and AN15 as Outputs */

    /* Increment active row and reset timer ASAP if more rows to scan */
    if (++ActiveRow < NUM_ROWS)
    {   /* Start next scan sequence if not at end of rows */
        TIMER3Start();
	    /* Drive next row active to initialise for next scan */
    	LATD = 1 << (ActiveRow); 
	}
	else
	{   /* Must be last row in this set so... */
	    /* Drive rows low to discharge */
    	LATD = 0; 
	}

    /* Use 2 buffers to allow time for processing column data */
    /* Always process the last buffer filed */
	if(DmaBuffer == 0)
	{
		ProcessADCSamples(&BufferA[0], (ActiveRow - 1));
	}
	else
	{
		ProcessADCSamples(&BufferB[0], (ActiveRow - 1));
	}

    if (ActiveRow >= NUM_ROWS)
	{
		/* End of full keyboard scan  */
        ActiveRow = 0;
        /*  Pointer for foreground task should point to the KeyboardScan just filled */
        NextKeyboardScan = &KeyboardScan[FillScan][0];
        /*  Fill other Keyboardscan buffer  */
        FillScan ^= 1;
    }
	
	DmaBuffer ^= 1;			/* Swap Buffers */

	IFS0bits.DMA0IF = 0;	/* Clear the DMA0 Interrupt Flag*/
}



/*****************************************************************************
*
* Name:    ProcessADCSamples
*
* Description: 
*
*   
*
*****************************************************************************/

void ProcessADCSamples(unsigned int * AdcBuffer, unsigned int Row)
{
	tRowScan *ptr;

	/* FillScan value is index to the buffer to be used next to fill by the ADC so the 
       alternative buffer must be the one filled last */

	ptr = &KeyboardScan[FillScan][Row];

	ptr->Col00 = (AdcBuffer[COL00] > AdcThreshold[Row]);
	ptr->Col01 = (AdcBuffer[COL01] > AdcThreshold[Row]);
	ptr->Col02 = (AdcBuffer[COL02] > AdcThreshold[Row]);
	ptr->Col03 = (AdcBuffer[COL03] > AdcThreshold[Row]);
	ptr->Col04 = (AdcBuffer[COL04] > AdcThreshold[Row]);
	ptr->Col05 = (AdcBuffer[COL05] > AdcThreshold[Row]);
	ptr->Col06 = (AdcBuffer[COL06] > AdcThreshold[Row]);
	ptr->Col07 = (AdcBuffer[COL07] > AdcThreshold[Row]);
	ptr->Col08 = (AdcBuffer[COL08] > AdcThreshold[Row]);
	ptr->Col09 = (AdcBuffer[COL09] > AdcThreshold[Row]);
	ptr->Col10 = (AdcBuffer[COL10] > AdcThreshold[Row]);
	ptr->Col11 = (AdcBuffer[COL11] > AdcThreshold[Row]);
	ptr->Col12 = (AdcBuffer[COL12] > AdcThreshold[Row]);
	ptr->Col13 = (AdcBuffer[COL13] > AdcThreshold[Row]);

}

    




