/******************************************************************************
* Filename          : Keybrd.h
*
* Module            : 
*
* Author            : Val Tarrant/John Thorne
* Description       : This file is the header file for keybrd.c 
*                     HT-MINI MIDI Keyboard 
*	
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   1.0     30Jan2008   Val Tarrant     First Issue
*
* Copyright         : Cambridge Micro Engineering, Jan 2008
******************************************************************************/
#ifndef __KEYBRD_H__
    #define __KEYBRD_H__

/*****************************************************************************
* Required includes
*****************************************************************************/

/*****************************************************************************
* Global Definitions
*****************************************************************************/

enum 
{
    ROW00, 
    ROW01, 
    ROW02,  
    ROW03, 
    ROW04, 
    ROW05, 
    ROW06, 
    ROW07, 
    ROW08, 
    ROW09, 
    ROW10, 
    ROW11, 
    ROW12, 
    ROW13,
    NUM_ROWS
};


enum 
{
    COL00, 
    COL01, 
    COL02,  
    COL03, 
    COL04, 
    COL05, 
    COL06, 
    COL07, 
    COL08, 
    COL09, 
    COL10, 
    COL11, 
    COL12, 
    COL13,
    NUM_COLUMNS
};

#define  NUM_SWITCHES_PER_KEY 2  /* Number of switches per key */

#define  NUM_KEYS ((NUM_ROWS/NUM_SWITCHES_PER_KEY) * (NUM_COLUMNS))
#define  NUM_SWITCHES (NUM_SWITCHES_PER_KEY * NUM_KEYS)

#define  SCANS_PER_ROW 2    /* Number of ADC scans per row required */

#define MAX_SCAN_COUNT 200

typedef union uRowScan
{
    unsigned _ColumnData;
    struct
    {
        unsigned Col00 : 1;
        unsigned Col01 : 1;
        unsigned Col02 : 1;
        unsigned Col03 : 1;
        unsigned Col04 : 1;
        unsigned Col05 : 1;
        unsigned Col06 : 1;
        unsigned Col07 : 1;
        unsigned Col08 : 1;
        unsigned Col09 : 1;
        unsigned Col10 : 1;
        unsigned Col11 : 1;
        unsigned Col12 : 1;
        unsigned Col13 : 1;
        unsigned Spare1: 1;
        unsigned Spare2: 1;
    };
} tRowScan;  

typedef enum eKeyStates
{
	KEY_STATE00=0,		/* Both switches up */
	KEY_STATE01,
	KEY_STATE02, 
	KEY_STATE03,		/* Both switches down */
	KEY_STATE04,
	KEY_STATE05,
	NUM_KEY_STATES
} tKeyStates;

typedef struct sKeyStatus
{
	tKeyStates	State;
	unsigned char Count;
    unsigned char MidiCode;
} tKeyStatus;

//typedef enum
//{
//    KEYBOARD_APP,
//    KEYBOARD_TEST
//} tKeyboardState;

typedef enum
{
    KEYBOARD_MODE_MIDI,
    KEYBOARD_MODE_ALT,
    KEYBOARD_MODE_TEST,
} tKeyboardMode;

typedef enum
{
    SWITCH_T_UP,
    SWITCH_T_DOWN,
    NUM_T_SWITCHES
} tTSwitch;

#define SWITCH_PRESSED 0
#define SWITCH_RELEASED 1



/*****************************************************************************
* Imported definitions
*****************************************************************************/

/*****************************************************************************
* Global variables
*****************************************************************************/

extern tRowScan * NextKeyboardScan;
extern tRowScan KeyboardScan[SCANS_PER_ROW][NUM_ROWS];

/*****************************************************************************
* Function prototype
*****************************************************************************/

void    KEYBRDInit (tKeyboardMode);
void    KEYBRDStateMachine(void);

#endif   /*  __KEYBRD_H__*/

/*****************************************************************************
*   End of file
*****************************************************************************/
