/*****************************************************************************
* Filename          : Timers.c $workfile: $
*
* Module            : 
*
* Author            : Val Tarrant
* Description       : This module holds initialisation functions 
*                     for hardware timers based on dsPIC 33fj128gp310 
*	
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   1.0     30Jan2008   Val Tarrant     First Issue
*
*   $Log: timers.c,v $
*   Revision 1.1  2009/09/22 15:55:13  rt
*   Initial commit to CVS.
*   Originally version 00.00.20
*   Committed on the Free edition of March Hare Software CVSNT Server.
*   Upgrade to CVS Suite for more features and support:
*   http://march-hare.com/cvsnt/
*
*
*
* Copyright         : Cambridge Micro Engineering, Jan 2008
*
******************************************************************************/

/*****************************************************************************
* Library Includes
*****************************************************************************/

#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#include <libpic30.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#include <libpic24.h>
#endif

/*****************************************************************************
* Project Includes
*****************************************************************************/
#include	"adcDrv1.h"
#include	"timers.h"
#include	"serial.h"
/******************************************************************************
*   Local Constants
******************************************************************************/

#define IRQ_PRIORITY_TIMER      (1)

/*****************************************************************************
*   Local Variables
******************************************************************************/


/*****************************************************************************
*   Global Variables
******************************************************************************/


/*****************************************************************************
*   Local Function Prototypes
*****************************************************************************/


/*****************************************************************************
*
* Name:    TIMER1Init
*
* Description: 
*          
* Description: Initialise Timer 1 to utilise the Tosc oscillator for 1 ms tick
*				No synchronisation, no prescale, no gate no stop in idle
*
*
*****************************************************************************/
void TIMER1Init (void)
{
	/* ensure Timer 1 is in reset state */
    T1CON = 0;
	TMR1 = 0x0000;

    T1CONbits.TCKPS = 0;    /* Divide by 1 */

	/* reset Timer 1 interrupt flag */
    IFS0bits.T1IF = 0;
    IEC0bits.T1IE = 0;

 	/* set Timer1 interrupt priority level */
    IPC0bits.T1IP = IRQ_PRIORITY_TIMER;

	/* enable Timer 1 interrupt */
    IEC0bits.T1IE = 1;

	/* set Timer 1 period register */
    PR1 = 39500;

    T1CONbits.TCS = 0;
    /* enable Timer 1 and start the count */
    T1CONbits.TON = 1;
}

/*****************************************************************************
*
* Name:    TIMER1Reset
*
* Description: 
*          
* Description: Initialise Timer 1 to utilise the Tosc oscillator for 1 ms tick
*				No synchronisation, no prescale, no gate no stop in idle
*
*
*****************************************************************************/
void TIMER1Reset (void)
{
	/* ensure Timer 1 is in reset state */
    T1CON = 0;
	TMR1 = 0x0000;

	/* reset Timer 1 interrupt flag */
    IFS0bits.T1IF = 0;
    IEC0bits.T1IE = 0;

}



/*****************************************************************************
*
* Name:    TIMER1Interrupt
*
* Description:  1 ms int Code 
*               Fast context save (using push.s and pop.s)
*
*****************************************************************************/
void __attribute__((interrupt, shadow, no_auto_psv)) _T1Interrupt(void)
{
    /* Start ADC */
    startAdc1();

	/* clear flag */
	IFS0bits.T1IF = 0;
        
}

/*=============================================================================  
Timer 3 is setup to time-out every ~5 microseconds (200Khz Rate). As a result, the module 
will stop sampling and trigger a conversion on every Timer3 time-out, i.e., Ts=5us. 
Timer 3 sampling at just under 5 microseconds  (4.80) to allow complete ADC scan 
i.e. 14 rows (7 swtiches 2 rows each switch) of 14 columns to complete in under 1 ms

=============================================================================*/
void TIMER3Init(void) 
{
	TMR3 = 0x0000;
	PR3  = 182;			    // Trigger ADC1 every ~5usec
	IFS0bits.T3IF = 0;		// Clear Timer 3 interrupt
	IEC0bits.T3IE = 0;		// Disable Timer 3 interrupt
	T3CONbits.TON = 1;		// Start Timer 3
}

void TIMER3Reset(void) 
{
	TMR3 = 0x0000;
	PR3  = 0;			    // Trigger ADC1 every ~5usec
	IFS0bits.T3IF = 0;		// Clear Timer 3 interrupt
	IEC0bits.T3IE = 0;		// Disable Timer 3 interrupt
	T3CONbits.TON = 0;		// Start Timer 3
}



/*=============================================================================  
Timer 3 needs to be stopped after dma int 
=============================================================================*/
void TIMER3Stop(void) 
{
	IFS0bits.T3IF = 0;		// Clear Timer 3 interrupt
	IEC0bits.T3IE = 0;		// Disable Timer 3 interrupt
	T3CONbits.TON = 0;		// Start Timer 3
}

/*=============================================================================  
Timer 3 needs to be restarted on 1 ms tick
=============================================================================*/

void TIMER3Start(void) 
{
	TMR3 = 0x0000;
	IFS0bits.T3IF = 0;		// Clear Timer 3 interrupt
	T3CONbits.TON = 1;		// Start Timer 3
}
