/**
**  @file       midi.h
**  @brief      MIDI definitions
**
**  Copyright (c) 2008, Twin Technologies Ltd
*/
#ifndef MIDI_H
#define MIDI_H

/*-------------------------------------------------------------------------*/
/*                                                                         */
/* Type definitions                                                        */
/*                                                                         */
/*-------------------------------------------------------------------------*/

/* Bitmasks */
#define MIDI_STATUS_BIT             (0x80)  /**< Indicates a MIDI status byte */
#define MIDI_MESSAGE_MASK           (0xf0)  /**< Extract message mask */
#define MIDI_MESSAGE_SHIFT             (4)  /**< Extract message shift*/
#define MIDI_SUBMESSAGE_MASK        (0x0f)  /**< Extract submessage mask */
#define MIDI_SUBMESSAGE_SHIFT          (0)  /**< Extract submessage shift */

#define MIDI_MESSAGE_SYSTEM         (0xf0)  /**< Base value for system messages */
#define MIDI_STATUS_REALTIME        (0x08)  /**< Real time system messages */
#define MIDI_STATUS_BOOTLOAD		(0x09)  /**< Bootload command */

/* MIDI status values */
#define MIDI_STATUS_NOTE_OFF        (0x80)
#define MIDI_STATUS_NOTE_ON         (0x90)
#define MIDI_STATUS_AFTERTOUCH      (0xa0)
#define MIDI_STATUS_CONTROL         (0xb0)
#define MIDI_STATUS_PROGRAM         (0xc0)
#define MIDI_STATUS_PRESSURE        (0xd0)
#define MIDI_STATUS_PITCHWHEEL      (0xe0)
#define MIDI_STATUS_SYSTEM          (0xf0)  /**< General grouping of system commands */
#define MIDI_STATUS_SYSTEM_RT       (0xf8)  /**< General grouping of real-time system commands */

/* System common messages */
#define MIDI_STATUS_SYSEX           (0xf0)
#define MIDI_STATUS_MTC             (0xf1)
#define MIDI_STATUS_SONG_POSITION   (0xf2)
#define MIDI_STATUS_SONG_SELECT     (0xf3)
#define MIDI_STATUS_F4              (0xf4)
#define MIDI_STATUS_F5              (0xf5)
#define MIDI_STATUS_TUNE_REQUEST    (0xf6)
#define MIDI_STATUS_END_SYSEX       (0xf7)

/* Real time system messages */
#define MIDI_STATUS_TIMING_CLOCK    (0xf8)
#define MIDI_STATUS_F9              (0xf9)	/**< Allocated to bootload command for USB software */
#define MIDI_STATUS_START           (0xfa)
#define MIDI_STATUS_CONTINUE        (0xfb)
#define MIDI_STATUS_STOP            (0xfc)
#define MIDI_STATUS_FD              (0xfd)
#define MIDI_STATUS_ACTIVE_SENSING  (0xfe)
#define MIDI_STATUS_RESET           (0xff)

/*
** USB MIDI definitions
*/
#define USBMIDI_LENGTH              (4)     /**< Number of bytes in a USB MIDI packet */
#define USBMIDI_CIN_MASK            (0x0f)
#define USBMIDI_CIN_SHIFT           (0)
#define USBMIDI_CABLE_MASK          (0xf0)
#define USBMIDI_CABLE_SHIFT         (4)

#define USB_MIDI_DATA_OFFSET        (1)     /**< Offset of data within USBMIDI packet */

#define USBMIDI_CIN_SINGLE          (0x0f)
#define USBMIDI_CIN_SYS_START       (0x04)
#define USBMIDI_CIN_SYS_END1        (0x05)
#define USBMIDI_CIN_SYS_END2        (0x06)
#define USBMIDI_CIN_SYS_END3        (0x07)

/*-------------------------------------------------------------------------*/
/*                                                                         */
/* External function interface                                             */
/*                                                                         */
/*-------------------------------------------------------------------------*/

#endif
